/*-----------------------------------------------------------------*/
/*! 
  \file calcephinternal.h 
  \brief private API for calceph library
         access and interpolate INPOP and JPL Ephemeris data.

  \author  M. Gastineau 
           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 

   Copyright, 2008,2009, 2010, 2011, CNRS
   email of the author : gastineau@imcce.fr
 
 History :
  \bug M. Gastineau 01/03/05 : thread-safe version 
  \bug M. Gastineau 11/05/06 : support DE411 and DE414
  \bug M. Gastineau 22/07/08 : support TT-TDB pour INPOP
  \bug M. Gastineau 17/10/08 : support DE421
  \note M. Gastineau 04/05/10 : support DE423
  \note M. Gastineau 12/10/10 : custom error handler
  \note M. Gastineau 25/11/10 : fix warning with sun studio compiler
  \note M. Gastineau 02/02/11 : fix warning with oracle studio compiler 12
  \bug  M. Gastineau 16/02/11 : fix packing with gcc 64 bits on solaris
*/
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* License  of this file :
 This file is "dual-licensed", you have to choose one  of the two licenses 
 below to apply on this file.
 
    CeCILL-C
    	The CeCILL-C license is close to the GNU LGPL.
    	( http://www.cecill.info/licences/Licence_CeCILL-C_V1-en.html )
 
 or CeCILL v2.0
      The CeCILL license is compatible with the GNU GPL.
      ( http://www.cecill.info/licences/Licence_CeCILL_V2-en.html )
 

This library is governed by the CeCILL-C or the CeCILL license under 
French law and abiding by the rules of distribution of free software.  
You can  use, modify and/ or redistribute the software under the terms 
of the CeCILL-C or CeCILL license as circulated by CEA, CNRS and INRIA  
at the following URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C or CeCILL license and that you accept its terms.
*/
/*-----------------------------------------------------------------*/


/*-----------------------------------------------------------------*/
/* private API */
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* constants */
/*-----------------------------------------------------------------*/
/* ephemeris numbers */
#define EPHEMERIS_DE200 200        
#define EPHEMERIS_DE403 403        
#define EPHEMERIS_DE405 405        
#define EPHEMERIS_DE406 406        
#define EPHEMERIS_DE411 411        
#define EPHEMERIS_DE414 414        
#define EPHEMERIS_DE418 418        
#define EPHEMERIS_DE421 421        
#define EPHEMERIS_DE423 423        
#define EPHEMERIS_INPOP 100        

/* size of the ephemeris blocks */
#define ARRAY_SIZE_DE200 826
#define ARRAY_SIZE_DE403 1018
#define ARRAY_SIZE_DE405 1018
#define ARRAY_SIZE_DE406 728
#define ARRAY_SIZE_DE411 1018
#define ARRAY_SIZE_DE414 1018
#define ARRAY_SIZE_DE418 1018
#define ARRAY_SIZE_DE421 1018
#define ARRAY_SIZE_DE423 1018


/* index of the bodies */
#define MERCURY 0
#define VENUS 1
#define EARTH 2                                   
#define MARS 3
#define JUPITER 4
#define SATURN 5
#define URANUS 6
#define NEPTUNE 7
#define PLUTO 8
#define MOON 9                                    
#define SUN 10
#define SS_BARY 11
#define EM_BARY 12
#define NUTATIONS 13
#define LIBRATIONS 14
#define TTMTDB 15

/*-----------------------------------------------------------------*/
/* structures */
/*-----------------------------------------------------------------*/


/*-----------------------------------------------------------------*/
/*! ephemeris state structure */
/*-----------------------------------------------------------------*/
typedef struct {
     treal Position[3]; /*!<  position vector */
     treal Velocity[3]; /*!<  velocity vector */
} stateType;

/*-------------------------------------------------------------------------*/
/*! content of binary header records                                       
 size of the structure and offset in bytes :
sizeof(struct recOneData)=2856
label=0
constName=252
timeData=2652
numConst=2676
AU=2680
EMRAT=2688
coeffPtr=2696
DENUM=2840
libratPtr=2844
*/
/*-------------------------------------------------------------------------*/

/* the next data structure must be packed : no alignment */
 /* push current alignment to stack */
#if !(defined(__GNUC__) && defined(sun))
#if defined(__IBMC__ )|| defined(__SUNPRO_C)
#pragma pack(1)
#else
#pragma pack(push,1) 
#endif
#endif

/*-------------------------------------------------------------------------*/
/*! content of the first record of the binary file*/
/*-------------------------------------------------------------------------*/
typedef struct  {
        char label[3][84];
        char constName[400][6];
        double timeData[3];
        int numConst;
        double AU;
        double EMRAT;
        int coeffPtr[12][3];
        int DENUM;
        int libratPtr[3];
        int recordsize;   /*!< size of the record in bytes (only in INPOP) */
        int TTmTDBPtr[3]; /*!< coefficients size/offset for TT-TDB (only in INPOP) */
}
#if defined(__GNUC__)
__attribute__((packed)) 
#endif
t_HeaderBlock; 

/* restore original alignment from stack */
#if !(defined(__GNUC__) && defined(sun))
#if defined(__SUNPRO_C)
#pragma pack()
#else
#pragma pack(pop)  
#endif
#endif

/*----------------------------------------------------------------------------*/
/* functions */
/*----------------------------------------------------------------------------*/
int calceph_interpol_PV(t_calcephbin *p_pbinfile, treal TimeJD0 , 
                           treal timediff, 
                           int Target , stateType *Planet );
void calceph_fatalerror(const char *format, ...);
#define fatalerror calceph_fatalerror

/*----------------------------------------------------------------------------*/
/* get constants */
/*----------------------------------------------------------------------------*/
double Ephemeris_getEMRAT(t_calcephbin *p_pbinfile);
double Ephemeris_getAU(t_calcephbin *p_pbinfile);
int    Ephemeris_getConstant(t_calcephbin *p_pbinfile, const char* name,
                             double *p_pdval);
